import Vue from 'vue'

// 注册一个全局的输入过滤特殊字符指令，用于过滤特殊字符 在元素上使用v-filter即可
Vue.directive('filter', {
  twoWay: true,
  bind: function () {
    this.handler = function () {
      var reg = new RegExp(/[ ,\`,\~,\!,\@,\#,\$,\%,\^,\*,\￥,\&,\\,\/,\?,\|,\:,\.,\<,\>,\{,\},\(,\),\',\;,\=,\"]/)
      this.el.value = this.el.value.replace(reg, '')
      this.set(this.el.value)
    }.bind(this)
    this.el.addEventListener('input', this.handler)
  },
  unbind: function () {
    this.el.removeEventListener('input', this.handler)
  }
})

// 注册一个全局自定义指令 next-el, 回车切换焦点，在元素上使用v-next-el="abc",给要切换到的元素上使用v-el: abc
Vue.directive('next-el', {
  bind: function () {
  },
  update: function (value) {
    this.handler = function () {
      // if (this.expression.includes('id')) {
      let el = null
      if (value) {
        el = document.getElementById(value.id)
      } else {
        el = this.vm.$els[this.expression]
      }
      if (el) {
        el.focus()
      }
    }.bind(this)
    this.el.addEventListener('keyup', function (event) {
      // console.log(event)
      if (event.keyCode === 13) {
        this.handler()
      }
    }.bind(this))
  },
  unbind: function () {
    this.el.removeEventListener('keyup', this.handler)
  }
})

// 保留小数位数，默认保留两位
Vue.directive('scale', {
  twoWay: true,
  update: function (value) {
    let fixed = 2
    let val = value
    let strs = this.expression
    if (typeof value === 'object') {
      val = value[0]
      fixed = value[1] || 2
      strs = this.expression.substring(1, this.expression.length - 1).split(',')[0]
    }
    if (val && val.toString().split('.')[1] && (val.toString().split('.')[1].length + 1) > fixed) {
      this.el.value = (val - 0).toFixed(fixed)
      eval(`this.vm.${strs} = ${this.el.value}`)
    }
  }
})

// 注册一个页面加载自动获取焦点指令，暂没有实现完成，调试开发中
Vue.directive('focus', {
  bind: function () {
    this.el.focus()
  }
})
