import Vue from 'vue'
import TreeList from '../stores/TreeList'
import co from 'co'

let openNode = function * (self, node) {
  yield self.toggle(node)
  self.openNode(node.children)
}

// 关闭自己及后代
function close (array, data) {
  // 移走自己
  array.$remove(data)
  if (!data.children) {
    return
  }

  // 对子进行递归调用
  for (let child of data.children) {
    close(array, child)
  }
}

// 展开自己及后代
function open (array, data, index, self) {
  array.splice(index + 1, 0, data)

  // 如果data为未展开或者没有子，直接返回
  if (!data.open || !data.children) {
    return
  }

  // 如果data的为展开，继续处理其子
  self.index = array.indexOf(data)
  for (let child of data.children) {
    open(array, child, self.index++ , self)
  }
}

// 依据data状态，对数据进行处理
function proc (model, data, self) {
  // 展开，添加子, 否则，移走
  if (data.open) {
    self.index = model.indexOf(data)
    for (let child of data.children) {
      open(model, child, self.index++, self)
    }
  } else {
    for (let child of data.children) {
      // 关闭子及其后代
      close(model, child)
    }
  }
  self.$dispatch('toggle')
}

// toggle方法的general函数
let toggleGen = function * (row, comp) {
  row.open = !row.open
  // 还没有加载子，调用加载子的过程
  if (row.open && !row.loaded) {
    yield comp.loadChild(row)
    // 把数据转换成树节点
    row.children = TreeList.toTreeNode(row.children, row, row.level + 1)
  }
  proc(comp.toNode, row, comp)
}

export default {
  props: ['url', 'model'],
  created () {
    // 如果开始有数据，把数据转换成树节点
    if (this.model && this.model.length > 0) {
      this.toNode = TreeList.toTreeNode(this.model)
    }
  },
  // computed: {
  //   toNode () {
  //     if (this.model && this.model.length > 0) {
  //       let a = TreeList.toTreeNode(this.model)
  //       console.log(this.model)
  //       return a
  //     }
  //   }
  // },
  data () {
    return {
      toNode: [],
      index: null
    }
  },
  watch: {
    'model' () {
      if (this.model) {
        this.toNode = TreeList.toTreeNode(this.model)
      }
    }
  },
  methods: {
    toggle (node) {
      let gen = toggleGen(node, this)
      return co(gen)
    },

    // 看节点是否含子
    isFolder (data) {
      return data.size > 0
    },

    // 加载子节点，加载过，不重复加载
    loadChild (node) {
      return new Promise((resolve, reject) => {
        // 如果不需要从后台获取数据，直接返回。适用于开始已经给定数据的场合。
        if (node.children.length !== 0) {
          node.loaded = true
          resolve()
          return
        }

        // 发送加载数据请求
        this.$resetpost(
          this.url,
          {data: {id: node.id}},
          {resolveMsg: null}
        ).then((response) => {
          node.children = response.data
          node.loaded = true
          Vue.set(node, 'state', '成功')
          resolve()
        }).catch(() => {
          node.open = false
          Vue.set(node, 'state', '错误')
        })
      })
    },

    // 刷新子节点，加载过，也照样加载，加载后，自动展开节点
    refreshChild (node) {
      // 如果节点已经展开，先关闭
      if (node.open) {
        this.toggle(node)
      }
      // 发送加载数据请求
      return this.$resetpost(
        this.url,
        {data: {id: node.id}},
        {resolveMsg: null}
      ).then((response) => {
        node.children = response.data
        node.loaded = true
        Vue.set(node, 'state', '成功')
        // 把加载的结果转换成树节点
        node.children = TreeList.toTreeNode(node.children, node, node.level + 1)
        // 纠正父节点size
        node.size = node.children.length
        // 重新展开节点
        this.toggle(node)
      }).catch(() => {
        node.open = false
        Vue.set(node, 'state', '错误')
      })
    },

    // 展开所有已经加载数据的树节点
    async openAll () {
      for (let node of this.toNode) {
        await this.openChild(node)
      }
    },

    // 展开某个节点的所有未展开的节点
    async openChild (node) {
      // 如果节点没有展开且有子，展开节点
      if (!node.open && node.children && node.children.length > 0) {
        await this.toggle(node)
      }
      // 递归调用子节点展开
      for (let child of node.children) {
        await this.openChild(child)
      }
    },

    // 删除节点
    remove (url, node) {
      Vue.remove(url, node).then(() => {
        // 从model移除
        this.model.$remove(node)
        // 如果有父，从父中移除
        if (node.parent) {
          node.parent.children.$remove(node)
          // 纠正父节点size
          node.parent.size = node.parent.children.length
        }
        // 删除以后，当前选中项变空
        if (this.select) {
          this.select(null)
        }
      })
    },

    // 展开节点
    openNode (nodes = this.toNode) {
      nodes.forEach((node) => {
        if (node.children.length > 0) {
          this.$nextTick(() => {
            let gen = openNode(this, node)
            co(gen)
          })
        }
      })
    },

    // 添加子节点
    addNode(node, data) {
      // 将数据转成节点
      let toChild = TreeList.toTreeNode([data], node, node.level+1)
      // 添加节点
      toChild.forEach((item, index) => {
        node.children.push(item)
      })
    },
    // defaultSp (arr) {
    //   console.log(arr)
    //   arr.forEach((row) => {
    //     let gen = toggleGen(row, this)
    //     co(gen)
    //     if (row.children) {
    //       this.defaultSp(row.children)
    //     }
    //   })
    // }
  }
}
