import Vue from 'vue'
import co from 'co'
import cryptJS from './EncryptUtil'
import CryptoJS from 'crypto-js'
import { GanShenMe } from '../util/Util'

// post执行过程
let postGen = function * (method, url, data, {warnMsg, resolveMsg, rejectMsg, type, silent, aoteEncrypt, encryKey},self) {
  Vue.set(self, 'silent', silent)
  if (self.isBusy === false) {
    self.isBusy = true
    // 如果需要提醒，显示提醒框
    let msgRet = 'confirm'
    if (warnMsg) {
      msgRet = yield Vue.showMessage(warnMsg, ['confirm', 'cancel'])
    }
    if (msgRet === 'confirm') {
      try {
        self.isEnd = false
        let dataRet = yield self.http(method, url, data, type, aoteEncrypt, encryKey)
        self.isEnd = true
        if (resolveMsg) {
          // 提交成功后，提醒提交成功
          // yield Vue.showMessage(resolveMsg)
          Vue.showAlert(resolveMsg, 'success', 2000)
        }
        // Vue.console.info(`http返回: ${JSON.stringify(dataRet)}`, 'HttpReset')
        self.isBusy = false
        self.silent = false
        return dataRet
      } catch (error) {
        self.isEnd = true
        if (rejectMsg) {
          // 提交失败后，提醒提交失败
          // yield Vue.showMessage(`${rejectMsg}错误状态：${error.status}，错误内容：${error.data}`)
          Vue.showAlert(`${rejectMsg}错误状态：${error.status}，错误内容：${error.data}`, 'danger', 0)
        }
        Vue.console.info(`http异常: ${JSON.stringify(error)}`, 'HttpReset')
        self.isBusy = false
        self.silent = false
        throw error
      }
    } else if (msgRet === 'cancel') {
      Vue.console.info('http请求: 用户按下了取消按钮，未提交请求', 'HttpReset')
      // 按下取消按钮，返回取消按钮按下状态
      let error = {status: 301, data: '用户按下了取消按钮，未提交请求'}
      self.isBusy = false
      self.silent = false
      throw error
    }
  } else {
    let error = {status: 603, data: '重复提交'}
    Vue.console.info(`http异常: ${JSON.stringify(error)}`, 'HttpReset')
    throw error
  }
}

export default class HttpReset {
  constructor () {
    this.xhr = new XMLHttpRequest()
    // 设置超时时间, 毫秒
    // this.xhr.timeout = 10000
    Vue.set(this, 'isEnd', true)
    Vue.set(this, 'isBusy', false)
  }

  load (method, url, data,{
    warnMsg = null,
    resolveMsg = '恭喜！提交成功！',
    rejectMsg = '提交错误！',
    type = 'json',
    silent = false,
    aoteEncrypt = aoteEncrypt ? aoteEncrypt : Vue.mmType,
    encryKey = ''
  } = {}, time) {
    if (this.xhr.timeout <= 0) {
      this.xhr.timeout = 0
    } else {
      this.xhr.timeout = (time ? time : 20000)
    }
    let gen = postGen(method, url, data, {warnMsg, resolveMsg, rejectMsg, type, silent, aoteEncrypt, encryKey}, this)
    return co(gen)
  }

  // 处理http请求
  // type: 返回数据类型，默认为json
  http (method, url, body, type = 'json', aoteEncrypt, encryKey) {
    if(aoteEncrypt){
      if(aoteEncrypt=='AES'){
        var data = {}
        data.aoteEncrypt = aoteEncrypt
        data.data = cryptJS.AESEncrypt(JSON.stringify(body), encryKey)
        body = data
      }
      if (aoteEncrypt == 'nginx' && url.indexOf("127.0.0.1") === -1) {
        let key = CryptoJS.enc.Utf8.parse(GanShenMe(Vue.HWQMM));
        if((typeof body) === 'string') {
          let srcs = CryptoJS.enc.Utf8.parse(body);
          body = CryptoJS.AES.encrypt(srcs, key, {mode:CryptoJS.mode.ECB,padding: CryptoJS.pad.Pkcs7}).toString();
        } else {
          let srcs = CryptoJS.enc.Utf8.parse(JSON.stringify(body));
          body =CryptoJS.AES.encrypt(srcs, key, {mode:CryptoJS.mode.ECB,padding: CryptoJS.pad.Pkcs7}).toString();
        }
      }
    }
    return new Promise((resolve, reject) => {
      if (!body) {
        body = {}
      }
      let self = this
      this.xhr.onload = function () {
        let data = this.responseText
        if (this.status === 200) {
          //Vue.console.info(`http返回`, 'HttpReset')
          if(aoteEncrypt){
            if(aoteEncrypt=='AES'){
              data=cryptJS.AESDecrypt(data, encryKey)
            }
            if(aoteEncrypt == 'nginx' && url.indexOf("127.0.0.1") === -1){
              // 匹配到规则，进行$解$密$处$理$
              let key = CryptoJS.enc.Utf8.parse(GanShenMe(Vue.HWQMM));
              data = CryptoJS.AES.decrypt(this.responseText,key,{mode:CryptoJS.mode.ECB,padding: CryptoJS.pad.Pkcs7}).toString(CryptoJS.enc.Utf8)
            }
          }
          try {
            // 根据返回类型，处理返回值
            if (type === 'json') {
              data = JSON.parse(data)
            }
          } catch (error) {
            data = data
          }
          resolve({data: data})
        } else {
          Vue.console.warn(`http异常, 错误码：${this.status}, 错误信息：${this.response}`, 'HttpReset``')
          reject({status: this.status, data: this.response})
        }
      }
      this.xhr.onabort = function () {
        Vue.console.warn(`请求被终止！`, 'HttpReset')
        reject({status: 610, data: '请求被终止！'})
      }
      this.xhr.ontimeout = function () {
        Vue.console.warn(`请求超时!`, 'HttpReset')
        reject({status: 611, data: '请求超时'})
      }
      this.xhr.onerror = function () {
        Vue.console.error(`网络故障!`, 'HttpReset')
        reject({status: 612, data: '网络故障'})
      }

      Vue.console.info(`http请求: ${url}`, 'HttpReset')
      this.xhr.open(method, url, true)
      // 如果有jwt且没有跨域，设置jwt
      if (!(url.startsWith('http://127.0.0.1')) && Vue.$login && Vue.$login.jwt) {
        console.log('设置请求头: jwt=' + Vue.$login.jwt)
        this.xhr.setRequestHeader('Token', Vue.$login.jwt)
      }
      this.xhr.send((typeof body) === 'string' ? body : JSON.stringify(body))
    })
  }
}
